"""A module containing multiple filters used by template engine."""
from __future__ import annotations

import platform

from qdarktheme2 import __version__
from qdarktheme2._color import Color
from qdarktheme2._icon.svg import Svg
from qdarktheme2._util import analyze_version_str, get_cash_root_path, get_logger
from qdarktheme2.qtpy import __version__ as qt_version
from qdarktheme2.qtpy.qt_compat import QT_API

_logger = get_logger(__name__)

if qt_version is None:
    _logger.warning("Failed to detect Qt version. Load Qt version as the latest version.")
    _QT_VERSION = "10.0.0"  # Fairly future version for always setting latest version.
else:
    _QT_VERSION = qt_version

if QT_API is None:
    _QT_API = "PySide6"
    _logger.warning(f"Failed to detect Qt binding. Load Qt API as '{_QT_API}'.")
else:
    _QT_API = QT_API

if None in (qt_version, QT_API):
    _logger.warning(
        "Maybe you need to install qt-binding. "
        "Available Qt-binding packages: PySide6, PyQt6, PyQt5, PySide2."
    )


def _transform(color: Color, color_state: dict[str, float]) -> Color:
    if color_state.get("transparent"):
        color = color.transparent(color_state["transparent"])
    if color_state.get("darken"):
        color = color.darken(color_state["darken"])
    if color_state.get("lighten"):
        return color.lighten(color_state["lighten"])
    return color


def color(color_info: str | dict[str, str | dict], state: str | None = None) -> Color:
    """Filter for template engine. This filter convert color info data to color object."""
    if isinstance(color_info, str):
        return Color.from_hex(color_info)

    base_color_format: str = color_info["base"]  # type: ignore
    color = Color.from_hex(base_color_format)

    if state is None:
        return color

    transforms = color_info[state]
    return Color.from_hex(transforms) if isinstance(transforms, str) else _transform(color, transforms)


def palette_format(color: Color) -> str:
    """Filter for template engine. This filter convert color object to ARGB hex format.

    QPalette parser for hex only support ARGB hex format. color.Color class use RGB hex format.
    So we need to convert Color object to ARGB hex format.
    """
    return f"#{color.to_hex_argb()}"


def url(color: Color, id: str, rotate: int = 0) -> str:
    """Filter for template engine. This filter create url for svg and output svg file."""
    svg_path = get_cash_root_path(__version__) / f"{id}_{color._to_hex()}_{rotate}.svg"
    url = f"url({svg_path.as_posix()})"
    if svg_path.exists():
        return url
    svg = Svg(id).colored(color).rotate(rotate)
    svg_path.write_text(str(svg))
    return url


def env(
    text, value: str, version: str | None = None, qt: str | None = None, os: str | None = None
) -> str:
    """Filter for template engine. This filter output empty string when unexpected environment."""
    if version and not analyze_version_str(_QT_VERSION, version):
        return ""
    if qt and qt.lower() != _QT_API.lower():
        return ""
    if os and platform.system().lower() not in os.lower():
        return ""
    return value.replace("${}", str(text))


def corner(corner_shape: str, size: str) -> str:
    """Filter for template engine. This filter manage corner shape."""
    return size if corner_shape == "rounded" else "0"



"""The ui to show Qt standard icons."""
from __future__ import annotations

from qdarktheme2.qtpy.QtCore import Qt
from qdarktheme2.qtpy.QtWidgets import (
    QGridLayout,
    QScrollArea,
    QStyle,
    QToolButton,
    QVBoxLayout,
    QWidget,
)


class IconsUi:
    """The ui class to show Qt standard icons."""

    def setup_ui(self, win: QWidget) -> None:
        """Set up ui."""
        widget_container = QWidget()
        layout = QGridLayout(widget_container)
        standard_pixmap_names = sorted(
            attr for attr in dir(QStyle.StandardPixmap) if attr.startswith("SP_")
        )
        if len(standard_pixmap_names) == 0:
            standard_pixmap_names = sorted(attr for attr in dir(QStyle) if attr.startswith("SP_"))

        for i, name in enumerate(standard_pixmap_names):
            button = QToolButton()
            button.setText(f":{name}:")
            button.setToolButtonStyle(Qt.ToolButtonStyle.ToolButtonTextBesideIcon)

            pixmap = getattr(QStyle.StandardPixmap, name)
            icon = win.style().standardIcon(pixmap)
            button.setIcon(icon)
            layout.addWidget(button, int(i / 4), i % 4)

        scroll_area = QScrollArea()
        scroll_area.setWidget(widget_container)

        v_main_layout = QVBoxLayout(win)
        v_main_layout.addWidget(scroll_area)



"""Module loading QPalette."""
from __future__ import annotations

from functools import partial

from qdarktheme2._template.engine import Template


def q_palette(mk_template: partial[Template], color_map: dict[str, str | dict], for_stylesheet: bool):
    """Generate QPalette."""
    from qdarktheme2.qtpy.QtGui import QColor, QPalette

    def _mk_q_color(text: str):
        template = mk_template(text)
        color_format = template.render(color_map)
        return QColor(color_format)

    palette = QPalette()

    if not for_stylesheet:
        # base
        palette.setColor(QPalette.ColorRole.WindowText, _mk_q_color("{{ foreground|color|palette }}"))
        palette.setColor(
            QPalette.ColorRole.Button, _mk_q_color("{{ treeSectionHeader.background|color|palette }}")
        )
        palette.setColor(QPalette.ColorRole.ButtonText, _mk_q_color("{{ primary|color|palette }}"))
        palette.setColor(QPalette.ColorRole.Base, _mk_q_color("{{ background|color|palette }}"))
        palette.setColor(QPalette.ColorRole.Window, _mk_q_color("{{ background|color|palette }}"))
        palette.setColor(QPalette.ColorRole.Highlight, _mk_q_color("{{ primary|color|palette }}"))
        palette.setColor(
            QPalette.ColorRole.HighlightedText, _mk_q_color("{{ background|color|palette }}")
        )
        palette.setColor(
            QPalette.ColorRole.AlternateBase,
            _mk_q_color("{{ list.alternateBackground|color|palette }}"),
        )
        palette.setColor(
            QPalette.ColorRole.ToolTipBase, _mk_q_color('{{ background|color(state="popup")|palette }}')
        )
        palette.setColor(QPalette.ColorRole.ToolTipText, _mk_q_color("{{ foreground|color|palette }}"))
        if hasattr(QPalette.ColorRole, "Foreground"):
            palette.setColor(
                QPalette.ColorRole.Foreground,  # type: ignore
                _mk_q_color("{{ foreground|color|palette }}"),
            )

        palette.setColor(QPalette.ColorRole.Light, _mk_q_color("{{ border|color|palette }}"))
        palette.setColor(QPalette.ColorRole.Midlight, _mk_q_color("{{ border|color|palette }}"))
        palette.setColor(QPalette.ColorRole.Dark, _mk_q_color("{{ background|color|palette }}"))
        palette.setColor(QPalette.ColorRole.Mid, _mk_q_color("{{ border|color|palette }}"))
        palette.setColor(QPalette.ColorRole.Shadow, _mk_q_color("{{ border|color|palette }}"))

        # disabled
        palette.setColor(
            QPalette.ColorGroup.Disabled,
            QPalette.ColorRole.WindowText,
            _mk_q_color('{{ foreground|color(state="disabled")|palette }}'),
        )
        palette.setColor(
            QPalette.ColorGroup.Disabled,
            QPalette.ColorRole.ButtonText,
            _mk_q_color('{{ foreground|color(state="disabled")|palette }}'),
        )
        palette.setColor(
            QPalette.ColorGroup.Disabled,
            QPalette.ColorRole.Highlight,
            _mk_q_color('{{ foreground|color(state="disabledSelectionBackground")|palette }}'),
        )
        palette.setColor(
            QPalette.ColorGroup.Disabled,
            QPalette.ColorRole.HighlightedText,
            _mk_q_color('{{ foreground|color(state="disabled")|palette }}'),
        )

        # inactive
        palette.setColor(
            QPalette.ColorGroup.Inactive,
            QPalette.ColorRole.Highlight,
            _mk_q_color('{{ primary|color(state="list.inactiveSelectionBackground")|palette }}'),
        )
        palette.setColor(
            QPalette.ColorGroup.Inactive,
            QPalette.ColorRole.HighlightedText,
            _mk_q_color("{{ foreground|color|palette }}"),
        )

    palette.setColor(
        QPalette.ColorRole.Text, _mk_q_color('{{ foreground|color(state="icon")|palette }}')
    )
    palette.setColor(
        QPalette.ColorGroup.Disabled,
        QPalette.ColorRole.Text,
        _mk_q_color('{{ foreground|color(state="disabled")|palette }}'),
    )
    palette.setColor(QPalette.ColorRole.Link, _mk_q_color("{{ primary|color|palette }}"))
    palette.setColor(QPalette.ColorRole.LinkVisited, _mk_q_color("{{ linkVisited|color|palette }}"))
    if hasattr(QPalette.ColorRole, "PlaceholderText"):
        palette.setColor(
            QPalette.ColorRole.PlaceholderText,
            _mk_q_color('{{ foreground|color(state="input.placeholder")|palette }}'),
        )

    palette.setColor(
        QPalette.ColorGroup.Disabled,
        QPalette.ColorRole.Link,
        _mk_q_color('{{ foreground|color(state="disabledSelectionBackground")|palette }}'),
    )
    palette.setColor(
        QPalette.ColorGroup.Disabled,
        QPalette.ColorRole.LinkVisited,
        _mk_q_color('{{ foreground|color(state="disabled")|palette }}'),
    )

    return palette



"""Module setting up ui of frame window."""
from qdarktheme2.qtpy.QtGui import QIcon
from qdarktheme2.qtpy.QtWidgets import (
    QCalendarWidget,
    QCheckBox,
    QFrame,
    QGridLayout,
    QGroupBox,
    QHBoxLayout,
    QPushButton,
    QRadioButton,
    QScrollArea,
    QSpinBox,
    QToolButton,
    QVBoxLayout,
    QWidget,
)


class FrameUI:
    """The ui class of frame window."""

    def setup_ui(self, win: QWidget) -> None:
        """Set up ui."""
        # Widgets
        group_box = QGroupBox("frameShape = Box")
        group_panel = QGroupBox("frameShape = Panel")
        group_none = QGroupBox("frameShape = NoFrame")
        group_line = QGroupBox("frameShape = VLine HLine")
        frame_box, frame_panel, frame_none, frame_v_line, frame_h_line = (QFrame() for _ in range(5))

        # Setup widgets
        frame_box.setFrameShape(QFrame.Shape.Box)
        frame_panel.setFrameShape(QFrame.Shape.Panel)
        frame_none.setFrameShape(QFrame.Shape.NoFrame)
        frame_v_line.setFrameShape(QFrame.Shape.VLine)
        frame_h_line.setFrameShape(QFrame.Shape.HLine)

        # Layout
        for frame in (frame_box, frame_panel, frame_none):
            push_btn_flat = QPushButton("Push button(flat)")
            push_btn_flat.setFlat(True)
            tool_btn = QToolButton()
            tool_btn.setIcon(QIcon("icons:favorite_border_24dp.svg"))
            calender = QCalendarWidget()

            g_layout = QGridLayout(frame)
            g_layout.addWidget(QPushButton("Push button"), 0, 0)
            g_layout.addWidget(push_btn_flat, 0, 1)
            g_layout.addWidget(QSpinBox(), 1, 0)
            g_layout.addWidget(tool_btn, 1, 1)
            g_layout.addWidget(QRadioButton("Radio button"), 2, 0)
            g_layout.addWidget(QCheckBox("Check box"), 2, 1)
            g_layout.addWidget(calender, 3, 0, 1, 2)

        for group, frame in (
            (group_box, frame_box),
            (group_panel, frame_panel),
            (group_none, frame_none),
        ):
            v_layout = QVBoxLayout(group)
            v_layout.addWidget(frame)

        h_layout = QHBoxLayout(group_line)
        h_layout.addWidget(frame_v_line)
        h_layout.addWidget(frame_h_line)

        widget_container = QWidget()
        g_layout = QGridLayout(widget_container)
        g_layout.addWidget(group_box, 0, 0)
        g_layout.addWidget(group_panel, 0, 1)
        g_layout.addWidget(group_none, 1, 0)
        g_layout.addWidget(group_line, 1, 1)

        scroll_area = QScrollArea()
        scroll_area.setWidget(widget_container)

        v_main_layout = QVBoxLayout(win)
        v_main_layout.addWidget(scroll_area)



from __future__ import annotations

import darkdetect

from qdarktheme2 import _os_appearance
from qdarktheme2.qtpy.QtCore import QCoreApplication, QEvent, QObject, QThread, Signal


class OSThemeSwitchListener(QThread):
    """Listener to detect to change OS's theme."""

    sig_run = Signal(bool)
    _sig_listen_os_theme = Signal()

    def __init__(self, callable) -> None:
        """Initialize listener."""
        super().__init__()
        self.setProperty("is_running", True)
        self._theme = darkdetect.theme()
        self._accent = _os_appearance.accent()
        self.sig_run.connect(lambda state: self.setProperty("is_running", state))
        self._sig_listen_os_theme.connect(callable)

    def eventFilter(self, q_object: QObject, event: QEvent) -> bool:  # noqa: N802
        """Override QObject.eventFilter.

        This override is for Mac.
        Qt can listen to change OS's theme on only mac and changed QPalette automatically.
        So enable to listen to change OS's theme via palette change event.
        """
        if (
            self.property("is_running")
            and q_object == QCoreApplication.instance()
            and event.type() == QEvent.Type.ApplicationPaletteChange
        ):
            accent = _os_appearance.accent()
            theme = darkdetect.theme()
            if self._theme != theme or self._accent != accent:
                self._theme = theme
                self._accent = accent
                self._sig_listen_os_theme.emit()
                return True
        return super().eventFilter(q_object, event)

    def run(self) -> None:
        """Override QThread.run.

        This override is for except Mac.
        Qt cannot listen to change OS's theme on except Mac.
        Use ``darkdetect.listener`` to detect to change OS's theme.
        """
        darkdetect.listener(
            lambda theme: self.property("is_running") and self._sig_listen_os_theme.emit()
        )

    def kill(self) -> None:
        """Kill thread."""
        self.terminate()
        self.deleteLater()



"""Module for QtSvg."""
from qdarktheme2.qtpy.qt_compat import QT_API, qt_import_error

if QT_API is None:
    raise qt_import_error
if QT_API == "PySide6":
    from PySide6.QtSvg import *  # type: ignore  # noqa: F403
elif QT_API == "PyQt6":
    from PyQt6.QtSvg import *  # type: ignore  # noqa: F403
elif QT_API == "PyQt5":
    from PyQt5.QtSvg import *  # type: ignore  # noqa: F403
elif QT_API == "PySide2":
    from PySide2.QtSvg import *  # type: ignore  # noqa: F403



# -*- coding: utf-8 -*-

from matplotlib.backends.backend_qt import _BackendQT, NavigationToolbar2QT, FigureManagerQT
from matplotlib.backends.backend_qtagg import *
from matplotlib.backends.qt_editor import _formlayout
from matplotlib.backends.qt_compat import QtWidgets
from matplotlib.backends.qt_editor._formlayout import FormDialog
import qtawesome as qta
import qdarktheme2  # Always import after Qt


class FigureCanvasQTAgg2(FigureCanvasQTAgg):
    def __init__(self, figure=None, *args, **kwargs):
        super().__init__(figure, *args, **kwargs)
        qdarktheme2.setup_theme('light')
        # self.figure.set_facecolor('#ebebeb')
        self.figure.set_facecolor('#f8f9fa')


@_BackendQT.export
class _BackendQTAgg(_BackendQT):
    FigureCanvas = FigureCanvasQTAgg2

def fedit2(data, title="", comment="", icon=None, parent=None, apply=None):
    if QtWidgets.QApplication.startingUp():
        _app = QtWidgets.QApplication([])
    dialog = FormDialog(data, title, comment, icon, parent, apply)
    dialog.setStyleSheet("QDialog{background-color: #f8f9fa;}")  # light
    # dialog.setStyleSheet("QDialog{background-color: #202124;}")  # dark
    dialog.setWindowIcon(qta.icon('ph.chart-line'))

    if parent is not None:
        if hasattr(parent, "_fedit_dialog"):
            parent._fedit_dialog.close()
        parent._fedit_dialog = dialog

    dialog.show()


_formlayout.fedit = fedit2


class NV(NavigationToolbar2QT):
    def __init__(self, canvas, parent=None, coordinates=True):
        super().__init__(canvas, parent, coordinates)
        self.setStyleSheet("QToolBar {background : #f8f9fa};")  # light
        # self.setStyleSheet("QToolBar {background : #202124};")  # dark

        btn_reset_zoom = self.actions()[0]
        btn_reset_zoom.setIcon(qta.icon('ph.house'))

        btn_arrow_left = self.actions()[1]
        btn_arrow_left.setIcon(qta.icon('ph.arrow-left'))

        btn_arrow_right = self.actions()[2]
        btn_arrow_right.setIcon(qta.icon('ph.arrow-right'))

        btn_move = self.actions()[4]
        btn_move.setIcon(qta.icon('ph.arrows-out-cardinal'))

        btn_zoom = self.actions()[5]
        btn_zoom.setIcon(qta.icon('ph.magnifying-glass-plus'))

        btn_subplots = self.actions()[6]
        btn_subplots.setIcon(qta.icon('ph.sliders'))

        btn_customize = self.actions()[7]
        btn_customize.setIcon(qta.icon('ph.chart-line'))

        btn_save = self.actions()[9]
        btn_save.setIcon(qta.icon('ph.floppy-disk'))

        self.setMovable(False)


FigureManagerQT._toolbar2_class = NV



import platform


def _check_macos_supported_version():
    sys_ver = platform.mac_ver()[0]  # typically 10.14.2 or 12.3
    major = int(sys_ver.split(".")[0])
    if major < 10:
        return False
    if major >= 11:
        return True
    minor = int(sys_ver.split(".")[1])
    return minor >= 14


def _dummy_accent_detector() -> None:
    return None


def _select_accent_detector():
    if platform.system() == "Darwin":
        if _check_macos_supported_version():
            from qdarktheme2._os_appearance._accent._mac_detect import get_mac_accent

            return get_mac_accent
        return _dummy_accent_detector
    return _dummy_accent_detector


accent = _select_accent_detector()



"""Module for handling template text."""
from __future__ import annotations

import json
import re
from dataclasses import dataclass
from itertools import chain, zip_longest

from qdarktheme2._util import multi_replace


@dataclass(unsafe_hash=True, frozen=True)
class _Placeholder:
    match_text: str
    value: str | int | float
    filters: tuple[str]


class Template:
    """Class that handles template text like jinja2."""

    _PLACEHOLDER_RE = re.compile(r"{{.*?}}")
    _STRING_RE = re.compile(r"""('([^'\\]*(?:\\.[^'\\]*)*)'|"([^"\\]*(?:\\.[^"\\]*)*)")""", re.S)

    def __init__(self, text: str, filters: dict):
        """Initialize Template class."""
        self._target_text = text
        self._filters = filters

    @staticmethod
    def _to_py_value(text: str):
        try:
            return int(text)
        except ValueError:
            try:
                return float(text)
            except ValueError:
                return text

    @staticmethod
    def _parse_placeholders(text: str):
        placeholders: set[_Placeholder] = set()
        for match in re.finditer(Template._PLACEHOLDER_RE, text):
            match_text = match.group()
            contents, *filters = match_text.strip("{}").replace(" ", "").split("|")
            value = Template._to_py_value(contents)
            placeholders.add(_Placeholder(match_text, value, tuple(filters)))
        return placeholders

    def _run_filter(self, value: str | int | float, filter_text: str):
        contents = filter_text.split("(")
        if len(contents) == 1:
            return self._filters[contents[0]](value)

        filter_name, arg_text = contents
        py_strings = [match.group() for match in Template._STRING_RE.finditer(arg_text)]
        if len(py_strings) == 0:
            json_text = '{"' + arg_text.replace("=", '":').replace(",", ',"').replace(")", "}")
        else:
            py_strings_escaped = [re.escape(py_string) for py_string in py_strings]
            words = re.split("|".join(py_strings_escaped), arg_text)
            words = [word.replace("=", '":').replace(",", ',"').replace(")", "}") for word in words]
            json_text = '{"' + "".join(
                chain.from_iterable(zip_longest(words, py_strings, fillvalue=""))
            )
        arguments: dict = json.loads(json_text)
        return self._filters[filter_name](value, **arguments)

    def render(self, replacements: dict) -> str:
        """Render replacements."""
        placeholders = Template._parse_placeholders(self._target_text)
        new_replacements: dict[str, str] = {}
        for placeholder in placeholders:
            value = placeholder.value
            if type(value) is str and len(value) != 0:
                value = replacements.get(value)
            if value is None:
                raise AssertionError(
                    f"There is no replacements for: {placeholder.value} in {placeholder.match_text}"
                )
            for filter in placeholder.filters:
                value = self._run_filter(value, filter)
            new_replacements[placeholder.match_text] = str(value)
        return multi_replace(self._target_text, new_replacements)



"""Module allowing for `python -m qdarktheme2.widget_gallery`."""
import sys

import qdarktheme2
from qdarktheme2.qtpy.QtWidgets import QApplication
from qdarktheme2.widget_gallery.main_window import WidgetGallery

if __name__ == "__main__":
    qdarktheme2.enable_hi_dpi()
    app = QApplication(sys.argv)
    qdarktheme2.setup_theme("auto")
    win = WidgetGallery()
    win.show()
    app.exec()



"""Module for loading style data for Qt."""
from __future__ import annotations

import json
import shutil
from functools import partial

from qdarktheme2 import __version__, _os_appearance, _resources
from qdarktheme2._template import filter
from qdarktheme2._template.engine import Template
from qdarktheme2._util import get_cash_root_path, get_logger

_logger = get_logger(__name__)


def _detect_system_theme(default_theme: str) -> str:
    import darkdetect

    system_theme = darkdetect.theme()
    if system_theme is None:
        _logger.info(f'failed to detect system theme, qdarktheme2 use "{default_theme}" theme.')
        return default_theme
    return system_theme.lower()


def _color_values(theme: str) -> dict[str, str | dict]:
    try:
        return json.loads(_resources.colors.THEME_COLOR_VALUES[theme])
    except KeyError:
        raise ValueError(f'invalid argument, not a dark, light or auto: "{theme}"') from None


def _has_primary_color(custom_colors: dict[str, str | dict[str, str]], theme: str) -> bool:
    if any("primary" in color for color in custom_colors):
        return True
    custom_colors_with_theme = custom_colors.get(f"[{theme}]")
    if custom_colors_with_theme is None:
        return False
    if not isinstance(custom_colors_with_theme, dict):
        raise ValueError(
            "invalid value for argument custom_colors, not a dict type: "
            f'"{custom_colors_with_theme}" of "[{theme}]" key.'
        )
    return any("primary" in color for color in custom_colors_with_theme)


def _apply_os_accent_color(
    custom_colors: dict[str, str | dict[str, str]] | None, theme: str
) -> dict[str, str | dict[str, str]] | None:
    accent = _os_appearance.accent()
    if accent is None:
        return custom_colors
    try:
        accent_color = _resources.colors.ACCENT_COLORS[theme].get(accent)
    except KeyError:
        raise ValueError(f'invalid argument, not a dark, light or auto: "{theme}"') from None
    if accent_color is None:
        return custom_colors

    if custom_colors is None:
        return {"primary": accent_color}
    custom_colors = custom_colors.copy()
    if not _has_primary_color(custom_colors, theme):
        custom_colors["primary"] = accent_color
    return custom_colors


def _mix_theme_colors(custom_colors: dict[str, str | dict[str, str]], theme: str) -> dict[str, str]:
    colors = {id: color for id, color in custom_colors.items() if isinstance(color, str)}
    custom_colors_with_theme = custom_colors.get(f"[{theme}]")
    if isinstance(custom_colors_with_theme, dict):
        colors.update(custom_colors_with_theme)
    elif isinstance(custom_colors_with_theme, str):
        raise ValueError(
            "invalid value for argument custom_colors, not a dict type: "
            f'"{custom_colors_with_theme}" of "[{theme}]" key.'
        )
    return colors


def _marge_colors(
    color_values: dict[str, str | dict], custom_colors: dict[str, str | dict[str, str]], theme: str
):
    for color_id, color in _mix_theme_colors(custom_colors, theme).items():
        try:
            parent_key, *child_keys = color_id.split(">")
            color_value = color_values[parent_key]
            if len(child_keys) > 1 or (isinstance(color_value, str) and len(child_keys) != 0):
                raise KeyError

            if isinstance(color_value, str):
                color_values[parent_key] = color
            else:
                child_key = "base" if len(child_keys) == 0 else child_keys[0]
                color_value[child_key]  # Check if child_key exists.
                color_value[child_key] = color
        except KeyError:
            raise KeyError(f'invalid color id for argument custom_colors: "{color_id}".') from None


def load_stylesheet(
    theme: str = "dark",
    corner_shape: str = "rounded",
    custom_colors: dict[str, str | dict[str, str]] | None = None,
    *,
    default_theme: str = "dark",
) -> str:
    """Load the style sheet which looks like flat design. There are `dark` and `light` theme.

    Args:
        theme: The theme name. There are `dark`, `light` and `auto`.
            If ``auto``, try to detect your OS's theme and accent (accent is only on Mac).
            If failed to detect OS's theme, use the default theme set in argument ``default_theme``.
            When primary color(``primary``) or primary child colors
            (such as ``primary>selection.background``) are set to custom_colors,
            disable to detect the accent.
        corner_shape: The corner shape. There are `rounded` and `sharp` shape.
        custom_colors: The custom color map. Overrides the default color for color id you set.
            Also you can customize a specific theme only. See example 6.
        default_theme: The default theme name.
            The theme set by this argument will be used when system theme detection fails.

    Raises:
        ValueError: If the arguments of this method is wrong.
        KeyError: If the color id of custom_colors is wrong.

    Returns:
        The stylesheet string for the given arguments.

    Examples:
        Set stylesheet to your Qt application.

        1. Dark Theme ::

            app = QApplication([])
            app.setStyleSheet(qdarktheme2.load_stylesheet())
            # or
            app.setStyleSheet(qdarktheme2.load_stylesheet("dark"))

        2. Light Theme ::

            app = QApplication([])
            app.setStyleSheet(qdarktheme2.load_stylesheet("light"))

        3. Automatic detection of system theme ::

            app = QApplication([])
            app.setStyleSheet(qdarktheme2.load_stylesheet("auto"))

        4. Sharp corner ::

            # Change corner shape to sharp.
            app = QApplication([])
            app.setStyleSheet(qdarktheme2.load_stylesheet(corner_shape="sharp"))

        5. Customize color ::

            app = QApplication([])
            app.setStyleSheet(qdarktheme2.load_stylesheet(custom_colors={"primary": "#D0BCFF"}))

        6. Customize a specific theme only ::

            app = QApplication([])
            app.setStyleSheet(
                qdarktheme2.load_stylesheet(
                    theme="auto",
                    custom_colors={
                        "[dark]": {
                            "primary": "#D0BCFF",
                        }
                    },
                )
            )
    """
    if theme == "auto":
        theme = _detect_system_theme(default_theme)
        custom_colors = _apply_os_accent_color(custom_colors, theme)
    color_values = _color_values(theme)
    if corner_shape not in ("rounded", "sharp"):
        raise ValueError(f'invalid argument, not a rounded or sharp: "{corner_shape}"')

    if custom_colors is not None:
        _marge_colors(color_values, custom_colors, theme)

    get_cash_root_path(__version__).mkdir(parents=True, exist_ok=True)

    stylesheet = _resources.stylesheets.TEMPLATE_STYLESHEET
    try:
        from qdarktheme2.qtpy.QtCore import QCoreApplication

        app = QCoreApplication.instance()
        if app is not None and not app.property("_qdarktheme_use_setup_style"):
            stylesheet += _resources.stylesheets.TEMPLATE_STANDARD_ICONS_STYLESHEET
    except Exception:  # noqa: PIE786
        pass

    # Build stylesheet
    template = Template(
        stylesheet,
        {"color": filter.color, "corner": filter.corner, "env": filter.env, "url": filter.url},
    )
    replacements = dict(color_values, **{"corner-shape": corner_shape})
    return template.render(replacements)


def clear_cache() -> None:
    """Clear the caches in system home path.

    PyQtDarkTheme build the caches of resources in the system home path.
    You can clear the caches by running this method.
    """
    try:
        cache_path = get_cash_root_path(__version__)
        shutil.rmtree(cache_path)
        _logger.info(f"The caches({cache_path}) has been deleted")
    except FileNotFoundError:
        _logger.info("There is no caches")


def load_palette(
    theme: str = "dark",
    custom_colors: dict[str, str | dict[str, str]] | None = None,
    *,
    default_theme: str = "dark",
    for_stylesheet: bool = False,
):
    """Load the QPalette for the dark or light theme.

    Args:
        theme: The theme name. There are `dark`, `light` and `auto`.
            If ``auto``, try to detect system theme.
            If failed to detect system theme, use the theme set in argument ``default_theme``.
        custom_colors: The custom color map. Overrides the default color for color id you set.
            Also you can customize a specific theme only. See example 5.
        default_theme: The default theme name.
            The theme set by this argument will be used when system theme detection fails.
        for_stylesheet: If True, only includes colors that cannot be set in stylesheets, such as
            ``link`` and ``placeholder``.

    Raises:
        TypeError: If the arg name of theme is wrong.
        KeyError: If the color id of custom_colors is wrong.

    Returns:
        QPalette: The QPalette for the given theme.

    Examples:
        Set QPalette to your Qt application.

        1. Dark Theme ::

            app = QApplication([])
            app.setPalette(qdarktheme2.load_palette())
            # or
            app.setPalette(qdarktheme2.load_palette("dark"))

        2. Light Theme ::

            app = QApplication([])
            app.setPalette(qdarktheme2.load_palette("light"))

        3. Automatic detection of system theme ::

            app = QApplication([])
            app.setPalette(qdarktheme2.load_palette("auto"))

        4. Customize color ::

            app = QApplication([])
            app.setPalette(custom_colors={"primary": "#D0BCFF"})

        5. Customize a specific theme only ::

            app = QApplication([])
            app.setStyleSheet(
                qdarktheme2.load_stylesheet(
                    theme="auto",
                    custom_colors={
                        "[dark]": {
                            "primary": "#D0BCFF",
                        }
                    },
                )
            )
    """
    if theme == "auto":
        theme = _detect_system_theme(default_theme)
    color_values = _color_values(theme)
    if custom_colors is not None:
        _marge_colors(color_values, custom_colors, theme)

    mk_template = partial(Template, filters={"color": filter.color, "palette": filter.palette_format})
    return _resources.palette.q_palette(mk_template, color_values, for_stylesheet)


def get_themes() -> tuple[str, ...]:
    """Return available theme names.

    Returns:
        Tuple of available theme names.
    """
    return _resources.THEMES



"""Module setting up ui of dock window."""
from __future__ import annotations

from qdarktheme2.qtpy.QtCore import Qt
from qdarktheme2.qtpy.QtWidgets import QDockWidget, QMainWindow, QTextEdit, QVBoxLayout, QWidget


class DockUI:
    """The ui class of dock window."""

    def setup_ui(self, win: QWidget) -> None:
        """Set up ui."""
        # Widgets
        left_dock = QDockWidget("Left dock")
        right_dock = QDockWidget("Right dock")
        top_dock = QDockWidget("Top dock")
        bottom_dock = QDockWidget("Bottom dock")

        # Setup widgets
        left_dock.setWidget(QTextEdit("This is the left widget."))
        right_dock.setWidget(QTextEdit("This is the right widget."))
        top_dock.setWidget(QTextEdit("This is the top widget."))
        bottom_dock.setWidget(QTextEdit("This is the bottom widget."))
        for dock in (left_dock, right_dock, top_dock, bottom_dock):
            dock.setAllowedAreas(
                Qt.DockWidgetArea.LeftDockWidgetArea
                | Qt.DockWidgetArea.RightDockWidgetArea
                | Qt.DockWidgetArea.BottomDockWidgetArea
                | Qt.DockWidgetArea.TopDockWidgetArea
            )

        # Layout
        main_win = QMainWindow()
        main_win.setCentralWidget(QTextEdit("This is the central widget."))
        main_win.addDockWidget(Qt.DockWidgetArea.LeftDockWidgetArea, left_dock)
        main_win.addDockWidget(Qt.DockWidgetArea.RightDockWidgetArea, right_dock)
        main_win.addDockWidget(Qt.DockWidgetArea.TopDockWidgetArea, top_dock)
        main_win.addDockWidget(Qt.DockWidgetArea.BottomDockWidgetArea, bottom_dock)

        layout = QVBoxLayout(win)
        layout.addWidget(main_win)
        layout.setContentsMargins(0, 0, 0, 0)



"""Module for QtGui."""
from qdarktheme2.qtpy.qt_compat import QT_API, qt_import_error

if QT_API is None:
    raise qt_import_error
if QT_API == "PySide6":
    from PySide6.QtGui import *  # type: ignore  # noqa: F403
elif QT_API == "PyQt6":
    from PyQt6.QtGui import *  # type: ignore  # noqa: F403
elif QT_API == "PyQt5":
    from PyQt5.QtGui import *  # type: ignore  # noqa: F403
    from PyQt5.QtWidgets import QAction, QActionGroup, QShortcut  # type: ignore
elif QT_API == "PySide2":
    from PySide2.QtGui import *  # type: ignore  # noqa: F403
    from PySide2.QtWidgets import QAction, QActionGroup, QShortcut  # type: ignore

if QT_API in ["PyQt5", "PySide2"]:
    QAction = QAction  # type: ignore  # noqa: SIM909
    QActionGroup = QActionGroup  # type: ignore  # noqa: SIM909
    QShortcut = QShortcut  # type: ignore  # noqa: SIM909



"""Module for detecting Qt version."""
from __future__ import annotations

from qdarktheme2.qtpy.qt_compat import QT_API

__version__: str | None = None
if QT_API == "PySide6":
    from PySide6.QtCore import qVersion  # type: ignore

    __version__ = qVersion()
elif QT_API == "PyQt6":
    from PyQt6.QtCore import qVersion  # type: ignore

    __version__ = qVersion()
elif QT_API == "PyQt5":
    from PyQt5.QtCore import qVersion  # type: ignore

    __version__ = qVersion()
elif QT_API == "PySide2":
    from PySide2.QtCore import qVersion  # type: ignore

    __version__ = qVersion()
